import tomllib
from pathlib import Path
from typing import Any

PYPROJ = "pyproject.toml"
EXTRA_PYPROJ = Path("_extra_pyproj")


def load_conf(path: Path) -> dict[str, Any]:
    with (path / PYPROJ).open("rb") as fp:
        return tomllib.load(fp)



from pathlib import Path
from typing import Any

from hatchling.builders.config import BuilderConfig
from hatchling.builders.hooks.plugin.interface import BuildHookInterface
from hatchling.plugin import hookimpl

from hatch_una import util


class UnaBuildHook(BuildHookInterface[BuilderConfig]):
    """
    Force-include all needed internal monorepo dependencies.
    """

    PLUGIN_NAME = "una-build"

    def initialize(self, version: str, build_data: dict[str, Any]) -> None:
        print("una: Injecting internal dependencies")

        # load the config for this package
        path = Path(self.root)
        conf = util.load_conf(path)
        name: str = conf["project"]["name"]

        try:
            int_deps: dict[str, str] = conf["tool"]["una"]["deps"]
        except KeyError as e:
            raise KeyError(
                f"Package '{name}' is missing '[tool.una.deps]' in pyproject.toml"
            ) from e

        if not int_deps:
            # this is fine, the package doesn't import anything internally
            return

        via_sdist = Path("PKG-INFO").exists()
        if via_sdist:
            # nothing to do as everything should already be included in sdist...
            return

        # make sure all int_deps exist
        found = [Path(k) for k in int_deps if (path / k).exists()]
        missing = set(int_deps) - set(str(p) for p in found)
        if len(missing) > 0:
            missing_str = ", ".join(missing)
            raise ValueError(f"Could not find these paths: {missing_str}")

        add_packages_pyproj = {
            str(f.parents[1] / util.PYPROJ): str(util.EXTRA_PYPROJ / f.name / util.PYPROJ)
            for f in found
        }

        build_data["force_include"] = {
            **build_data["force_include"],
            **int_deps,
            **add_packages_pyproj,
        }


@hookimpl
def hatch_register_build_hook() -> type[UnaBuildHook]:
    return UnaBuildHook



from pathlib import Path
from typing import Any

from hatchling.metadata.plugin.interface import MetadataHookInterface
from hatchling.plugin import hookimpl

from hatch_una import util


class UnaMetaHook(MetadataHookInterface):
    """
    Inject needed third-party dependencies into project.dependencies.
    """

    PLUGIN_NAME = "una-meta"

    def update(self, metadata: dict[str, Any]) -> None:
        print("una: Injecting transitive external dependencies")

        # load the config for this package
        path = Path(self.root)
        conf = util.load_conf(path)
        name: str = conf["project"]["name"]

        try:
            int_deps: dict[str, str] = conf["tool"]["una"]["deps"]
        except KeyError as e:
            raise KeyError(
                f"Package '{name}' is missing '[tool.una.deps]' in pyproject.toml"
            ) from e

        project_deps: list[str] = metadata.get("dependencies", [])
        project_deps = [d.strip().replace(" ", "") for d in project_deps]

        add_deps: list[str] = []
        for dep_path in int_deps:
            # In builds that do src -> sdist -> wheel, the needed pyproject.toml files
            # will have been copied into the sdist so they're available for the wheel build.
            # Here we check for both in order.
            dep_project_path = Path(dep_path).parents[1]
            extra_path = util.EXTRA_PYPROJ / Path(dep_path).name
            if dep_project_path.exists():
                use_path = dep_project_path
            elif extra_path.exists():
                use_path = extra_path
            else:
                raise ValueError(f"Could not find internal dependency at '{dep_path}'")

            # load all third-party dependencies from this internal dependency into the
            # project.dependencies table
            dep_conf = util.load_conf(use_path)
            try:
                dep_deps: list[str] = dep_conf["project"]["dependencies"]
            except KeyError as e:
                raise KeyError(f"No project.dependencies table for '{use_path}'")
            dep_deps = [d.strip().replace(" ", "") for d in dep_deps]
            add_deps.extend(dep_deps)

        metadata["dependencies"] = list(set(project_deps + add_deps))


@hookimpl
def hatch_register_metadata_hook() -> type[UnaMetaHook]:
    return UnaMetaHook
